package cn.com.infosec.crypto.digests;

import cn.com.infosec.crypto.ExtendedDigest;
import cn.com.infosec.util.Arrays;

public final class WhirlpoolDigest implements ExtendedDigest {
   private static final int BYTE_LENGTH = 64;
   private static final int DIGEST_LENGTH_BYTES = 64;
   private static final int ROUNDS = 10;
   private static final int REDUCTION_POLYNOMIAL = 285;
   private static final int[] SBOX = new int[]{24, 35, 198, 232, 135, 184, 1, 79, 54, 166, 210, 245, 121, 111, 145, 82, 96, 188, 155, 142, 163, 12, 123, 53, 29, 224, 215, 194, 46, 75, 254, 87, 21, 119, 55, 229, 159, 240, 74, 218, 88, 201, 41, 10, 177, 160, 107, 133, 189, 93, 16, 244, 203, 62, 5, 103, 228, 39, 65, 139, 167, 125, 149, 216, 251, 238, 124, 102, 221, 23, 71, 158, 202, 45, 191, 7, 173, 90, 131, 51, 99, 2, 170, 113, 200, 25, 73, 217, 242, 227, 91, 136, 154, 38, 50, 176, 233, 15, 213, 128, 190, 205, 52, 72, 255, 122, 144, 95, 32, 104, 26, 174, 180, 84, 147, 34, 100, 241, 115, 18, 64, 8, 195, 236, 219, 161, 141, 61, 151, 0, 207, 43, 118, 130, 214, 27, 181, 175, 106, 80, 69, 243, 48, 239, 63, 85, 162, 234, 101, 186, 47, 192, 222, 28, 253, 77, 146, 117, 6, 138, 178, 230, 14, 31, 98, 212, 168, 150, 249, 197, 37, 89, 132, 114, 57, 76, 94, 120, 56, 140, 209, 165, 226, 97, 179, 33, 156, 30, 67, 199, 252, 4, 81, 153, 109, 13, 250, 223, 126, 36, 59, 171, 206, 17, 143, 78, 183, 235, 60, 129, 148, 247, 185, 19, 44, 211, 231, 110, 196, 3, 86, 68, 127, 169, 42, 187, 193, 83, 220, 11, 157, 108, 49, 116, 246, 70, 172, 137, 20, 225, 22, 58, 105, 9, 112, 182, 208, 237, 204, 66, 152, 164, 40, 92, 248, 134};
   private static final long[] C0 = new long[256];
   private static final long[] C1 = new long[256];
   private static final long[] C2 = new long[256];
   private static final long[] C3 = new long[256];
   private static final long[] C4 = new long[256];
   private static final long[] C5 = new long[256];
   private static final long[] C6 = new long[256];
   private static final long[] C7 = new long[256];
   private static final int BITCOUNT_ARRAY_SIZE = 32;
   private static final short[] EIGHT;
   private final long[] _rc;
   private byte[] _buffer;
   private int _bufferPos;
   private short[] _bitCount;
   private long[] _hash;
   private long[] _K;
   private long[] _L;
   private long[] _block;
   private long[] _state;

   static {
      (EIGHT = new short[32])[31] = 8;
   }

   public WhirlpoolDigest() {
      this._rc = new long[11];
      this._buffer = new byte[64];
      this._bufferPos = 0;
      this._bitCount = new short[32];
      this._hash = new long[8];
      this._K = new long[8];
      this._L = new long[8];
      this._block = new long[8];
      this._state = new long[8];

      int var1;
      int var2;
      for(var1 = 0; var1 < 256; ++var1) {
         int var3;
         int var4;
         int var10001 = var4 = this.maskWithReductionPolynomial((var3 = this.maskWithReductionPolynomial((var2 = SBOX[var1]) << 1)) << 1);
         int var5 = var10001 ^ var2;
         int var6;
         int var7 = (var6 = this.maskWithReductionPolynomial(var10001 << 1)) ^ var2;
         C0[var1] = this.packIntoLong(var2, var2, var4, var2, var6, var5, var3, var7);
         C1[var1] = this.packIntoLong(var7, var2, var2, var4, var2, var6, var5, var3);
         C2[var1] = this.packIntoLong(var3, var7, var2, var2, var4, var2, var6, var5);
         C3[var1] = this.packIntoLong(var5, var3, var7, var2, var2, var4, var2, var6);
         C4[var1] = this.packIntoLong(var6, var5, var3, var7, var2, var2, var4, var2);
         C5[var1] = this.packIntoLong(var2, var6, var5, var3, var7, var2, var2, var4);
         C6[var1] = this.packIntoLong(var4, var2, var6, var5, var3, var7, var2, var2);
         C7[var1] = this.packIntoLong(var2, var4, var2, var6, var5, var3, var7, var2);
      }

      this._rc[0] = 0L;

      for(var1 = 1; var1 <= 10; ++var1) {
         var2 = (var1 - 1) * 8;
         this._rc[var1] = C0[var2] & -72057594037927936L ^ C1[var2 + 1] & 71776119061217280L ^ C2[var2 + 2] & 280375465082880L ^ C3[var2 + 3] & 1095216660480L ^ C4[var2 + 4] & 4278190080L ^ C5[var2 + 5] & 16711680L ^ C6[var2 + 6] & 65280L ^ C7[var2 + 7] & 255L;
      }

   }

   private long packIntoLong(int b7, int b6, int b5, int b4, int b3, int b2, int b1, int b0) {
      return (long)b7 << 56 ^ (long)b6 << 48 ^ (long)b5 << 40 ^ (long)b4 << 32 ^ (long)b3 << 24 ^ (long)b2 << 16 ^ (long)b1 << 8 ^ (long)b0;
   }

   private int maskWithReductionPolynomial(int input) {
      if ((long)input >= 256L) {
         input ^= 285;
      }

      return input;
   }

   public WhirlpoolDigest(WhirlpoolDigest originalDigest) {
      WhirlpoolDigest var10000 = originalDigest;
      WhirlpoolDigest var10001 = originalDigest;
      WhirlpoolDigest var10002 = originalDigest;
      WhirlpoolDigest var10003 = originalDigest;
      WhirlpoolDigest var10004 = originalDigest;
      WhirlpoolDigest var10005 = originalDigest;
      WhirlpoolDigest var10007 = originalDigest;
      WhirlpoolDigest var10008 = originalDigest;
      WhirlpoolDigest var10009 = originalDigest;
      super();
      long[] originalDigest1;
      this._rc = originalDigest1 = new long[11];
      this._buffer = new byte[64];
      this._bufferPos = 0;
      this._bitCount = new short[32];
      this._hash = new long[8];
      this._K = new long[8];
      this._L = new long[8];
      this._block = new long[8];
      this._state = new long[8];
      long[] var15 = var10009._rc;
      int var2 = originalDigest1.length;
      System.arraycopy(var15, 0, originalDigest1, 0, var2);
      byte[] var14 = var10008._buffer;
      byte[] originalDigest2;
      var2 = (originalDigest2 = this._buffer).length;
      System.arraycopy(var14, 0, originalDigest2, 0, var2);
      this._bufferPos = var10007._bufferPos;
      short[] var13 = var10005._bitCount;
      short[] originalDigest3;
      var2 = (originalDigest3 = this._bitCount).length;
      System.arraycopy(var13, 0, originalDigest3, 0, var2);
      long[] var12 = var10004._hash;
      var2 = (originalDigest1 = this._hash).length;
      System.arraycopy(var12, 0, originalDigest1, 0, var2);
      long[] var11 = var10003._K;
      var2 = (originalDigest1 = this._K).length;
      System.arraycopy(var11, 0, originalDigest1, 0, var2);
      long[] var10 = var10002._L;
      var2 = (originalDigest1 = this._L).length;
      System.arraycopy(var10, 0, originalDigest1, 0, var2);
      long[] var9 = var10001._block;
      var2 = (originalDigest1 = this._block).length;
      System.arraycopy(var9, 0, originalDigest1, 0, var2);
      long[] var8 = var10000._state;
      long[] var3;
      int originalDigest4 = (var3 = this._state).length;
      System.arraycopy(var8, 0, var3, 0, originalDigest4);
   }

   private void processFilledBuffer(byte[] in, int inOff) {
      for(int in1 = 0; in1 < this._state.length; ++in1) {
         byte[] inOff1 = this._buffer;
         this._block[in1] = this.bytesToLongFromBuffer(inOff1, in1 * 8);
      }

      this.processBlock();
      this._bufferPos = 0;
      Arrays.fill(this._buffer, (byte)0);
   }

   private long bytesToLongFromBuffer(byte[] buffer, int startPos) {
      return ((long)buffer[startPos + 0] & 255L) << 56 | ((long)buffer[startPos + 1] & 255L) << 48 | ((long)buffer[startPos + 2] & 255L) << 40 | ((long)buffer[startPos + 3] & 255L) << 32 | ((long)buffer[startPos + 4] & 255L) << 24 | ((long)buffer[startPos + 5] & 255L) << 16 | ((long)buffer[startPos + 6] & 255L) << 8 | (long)buffer[startPos + 7] & 255L;
   }

   private void convertLongToByteArray(long inputLong, byte[] outputArray, int offSet) {
      for(int var6 = 0; var6 < 8; ++var6) {
         int var5 = offSet + var6;
         outputArray[var5] = (byte)((int)(inputLong >> 56 - var6 * 8 & 255L));
      }

   }

   private void increment() {
      int var1 = 0;

      for(int var2 = this._bitCount.length - 1; var2 >= 0; --var2) {
         short[] var10000 = this._bitCount;
         int var10002 = (var10000[var2] & 255) + EIGHT[var2] + var1;
         var1 = var10002 >>> 8;
         var10000[var2] = (short)(var10002 & 255);
      }

   }

   private void finish() {
      byte[] var1 = this.copyBitLength();
      byte[] var2 = this._buffer;
      int var3;
      int var4;
      int var10000 = var4 = (var3 = this._bufferPos) + 1;
      this._bufferPos = var4;
      var2[var3] = (byte)(var2[var3] | 128);
      if (var10000 == var2.length) {
         this.processFilledBuffer(var2, 0);
      }

      if (this._bufferPos > 32) {
         while(this._bufferPos != 0) {
            this.update((byte)0);
         }
      }

      while(this._bufferPos <= 32) {
         this.update((byte)0);
      }

      WhirlpoolDigest var7 = this;
      WhirlpoolDigest var10001 = this;
      byte[] var10002 = var1;
      byte[] var5 = this._buffer;
      int var6 = var1.length;
      System.arraycopy(var10002, 0, var5, 32, var6);
      var7.processFilledBuffer(var10001._buffer, 0);
   }

   private byte[] copyBitLength() {
      byte var1 = 32;
      byte[] var2 = new byte[32];

      for(int var3 = 0; var3 < var1; ++var3) {
         var2[var3] = (byte)(this._bitCount[var3] & 255);
      }

      return var2;
   }

   public String getAlgorithmName() {
      return "Whirlpool";
   }

   public int getDigestSize() {
      return 64;
   }

   public int doFinal(byte[] out, int outOff) {
      this.finish();

      for(int var3 = 0; var3 < 8; ++var3) {
         long var4 = this._hash[var3];
         int var6 = outOff + var3 * 8;
         this.convertLongToByteArray(var4, out, var6);
      }

      this.reset();
      return this.getDigestSize();
   }

   public void reset() {
      this._bufferPos = 0;
      Arrays.fill(this._bitCount, (short)0);
      Arrays.fill(this._buffer, (byte)0);
      Arrays.fill(this._hash, 0L);
      Arrays.fill(this._K, 0L);
      Arrays.fill(this._L, 0L);
      Arrays.fill(this._block, 0L);
      Arrays.fill(this._state, 0L);
   }

   protected void processBlock() {
      int var1;
      for(var1 = 0; var1 < 8; ++var1) {
         long var10002 = this._block[var1];
         long var2;
         this._K[var1] = var2 = this._hash[var1];
         this._state[var1] = var10002 ^ var2;
      }

      byte var5 = 1;

      long[] var10000;
      for(int var6 = var5; var6 <= 10; var6 = var1) {
         long[] var10016;
         long[] var10018;
         long[] var3;
         long[] var10020;
         long[] var10022;
         long[] var10024;
         long[] var10;
         long[] var10026;
         long[] var10004;
         long[] var10006;
         long[] var10008;
         long[] var10010;
         long[] var10012;
         long[] var10014;
         for(var1 = 0; var1 < 8; ++var1) {
            var10000 = var3 = this._L;
            var10 = var3;
            var10004 = var3;
            var10006 = var3;
            var10008 = var3;
            var10010 = var3;
            var10012 = var3;
            var10014 = var3;
            var10016 = var3;
            var10018 = var3;
            var10020 = var3;
            var10022 = var3;
            var10024 = var3;
            var10026 = var3;
            var3[var1] = 0L;
            var3[var1] ^= C0[(int)((var3 = this._K)[var1 - 0 & 7] >>> 56) & 255];
            var10024[var1] = var10026[var1] ^ C1[(int)(var3[var1 - 1 & 7] >>> 48) & 255];
            var10020[var1] = var10022[var1] ^ C2[(int)(var3[var1 - 2 & 7] >>> 40) & 255];
            var10016[var1] = var10018[var1] ^ C3[(int)(var3[var1 - 3 & 7] >>> 32) & 255];
            var10012[var1] = var10014[var1] ^ C4[(int)(var3[var1 - 4 & 7] >>> 24) & 255];
            var10008[var1] = var10010[var1] ^ C5[(int)(var3[var1 - 5 & 7] >>> 16) & 255];
            var10004[var1] = var10006[var1] ^ C6[(int)(var3[var1 - 6 & 7] >>> 8) & 255];
            var10000[var1] = var10[var1] ^ C7[(int)var3[var1 - 7 & 7] & 255];
         }

         long[] var7;
         int var8 = (var7 = this._K).length;
         System.arraycopy(this._L, 0, var7, 0, var8);
         var10000 = this._K;
         long var9 = var10000[0] ^ this._rc[var6];
         var10000[0] = var9;

         for(var1 = 0; var1 < 8; ++var1) {
            var10000 = var3 = this._L;
            var10 = var3;
            var10004 = var3;
            var10006 = var3;
            var10008 = var3;
            var10010 = var3;
            var10012 = var3;
            var10014 = var3;
            var10016 = var3;
            var10018 = var3;
            var10020 = var3;
            var10022 = var3;
            var10024 = var3;
            var10026 = var3;
            var3[var1] = this._K[var1];
            var3[var1] ^= C0[(int)((var3 = this._state)[var1 - 0 & 7] >>> 56) & 255];
            var10024[var1] = var10026[var1] ^ C1[(int)(var3[var1 - 1 & 7] >>> 48) & 255];
            var10020[var1] = var10022[var1] ^ C2[(int)(var3[var1 - 2 & 7] >>> 40) & 255];
            var10016[var1] = var10018[var1] ^ C3[(int)(var3[var1 - 3 & 7] >>> 32) & 255];
            var10012[var1] = var10014[var1] ^ C4[(int)(var3[var1 - 4 & 7] >>> 24) & 255];
            var10008[var1] = var10010[var1] ^ C5[(int)(var3[var1 - 5 & 7] >>> 16) & 255];
            var10004[var1] = var10006[var1] ^ C6[(int)(var3[var1 - 6 & 7] >>> 8) & 255];
            var10000[var1] = var10[var1] ^ C7[(int)var3[var1 - 7 & 7] & 255];
         }

         var8 = (var7 = this._state).length;
         System.arraycopy(this._L, 0, var7, 0, var8);
         var1 = var6 + 1;
      }

      for(var1 = 0; var1 < 8; ++var1) {
         var10000 = this._hash;
         var10000[var1] ^= this._state[var1] ^ this._block[var1];
      }

   }

   public void update(byte in) {
      byte[] var2;
      int var3;
      (var2 = this._buffer)[var3 = this._bufferPos] = (byte)in;
      int var10000 = in = var3 + 1;
      this._bufferPos = in;
      if (var10000 == var2.length) {
         this.processFilledBuffer(var2, 0);
      }

      this.increment();
   }

   public void update(byte[] in, int inOff, int len) {
      while(len > 0) {
         this.update(in[inOff]);
         ++inOff;
         --len;
      }

   }

   public int getByteLength() {
      return 64;
   }
}
